--------------------------------------------------------------------------------
-- SubModule MulShifter
-- Created   4/8/2005 8:30:57 PM
--------------------------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Numeric_Std.all;

entity MulShifter is
   port (
     a        : in    std_logic_vector(35 downto 0);
     b        : out   std_logic_vector(47 downto 0);
     sel      : in    std_logic_vector (1 downto 0);
     CLK      : in    std_logic
   );
end MulShifter;
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
architecture RTL of MulShifter is

   constant mulwidth : integer := 18;
   constant width : integer := 48;
   constant drop : integer := 25;

begin
   registered_shifter: process (CLK) is
      variable s : std_logic_vector(width-1 downto 0);
   begin
      if rising_edge (CLK) then
      case sel is
         -- LSB * LSB: no shift, no sign extend, but drop
         when B"00" =>
--            if drop < 2*(mulwidth-1) then
               s(2*mulwidth-3-drop downto 0) :=a(2*mulwidth-3 downto drop);
               s(width-1 downto 2*mulwidth-2-drop) := (others => '0');
--            else
--               s(width-1 downto 0) := (others => '0');
--            end if;
         -- MSB * LSB or LSB * MSB: shift mulwitdh-1, sign extend, drop end or fill with zeros
         when B"01" =>
--            if drop >= mulwidth-1 then
               s(3*mulwidth-1-drop downto 0) := a(2*mulwidth-1 downto drop-mulwidth);
               s(width-1 downto 3*mulwidth-drop) := (others => a(2*mulwidth-1));
--            else
--               s(mulwidth-2-drop downto 0) := (others => '0');
--               s(3*mulwidth-2-drop downto mulwidth-1-drop) := a;
--               s(width-1 downto 3*mulwidth-1-drop) := (others => a(2*mulwidth-1));
--            end if;
         -- MSB * MSB: shift 2*(mulwitdh-1), sign extend, fill end with zeros or drop
         when B"10" | B"11" =>
--            if drop < 2*(mulwidth-1) then
               s(2*mulwidth-3-drop downto 0) := (others => '0');
               s(4*mulwidth-3-drop downto 2*mulwidth-2-drop) := a(2*mulwidth-1 downto 0);
               s(width-1 downto 4*mulwidth-2-drop) := (others => a(2*mulwidth-1));
--            else
--               s(4*mulwidth-3-drop downto 0) := a (2*mulwidth-1 downto drop-2*(mulwidth-1));
--               s(width-1 downto 4*mulwidth-2-drop) := (others => a(2*mulwidth-1));
--            end if;
         when others =>
            s(width-1 downto 0) := (others => '0');
      end case;
      b <= s;
      end if;
   end process registered_shifter;
end architecture RTL;
--------------------------------------------------------------------------------
